import 'dart:convert';
import 'package:http/http.dart' as http;

import './messages.dart';
import './groups.dart';

/// VK API class
class API {
  /// Base API request url
  String _baseUrl = 'https://api.vk.com/method';
  
  /// Group's or user's token
  String _token;

  /// API version
  String _version = '5.98';

  /// Messages class
  Messages messages;

  /// Groups class
  Groups groups;

  /// API
  API(String token) {
    this._token = token;

    this.messages = new Messages(token);
    this.groups = new Groups(token);
  }

  toString() {
    return 'API';
  }

  /// Call an API [method] with [params]
  Future<Map> call(String method, [Map<String, String> params]) async {
    Map body = <String, String>{
      'access_token': this._token,
      'v': this._version,
      ...?params,
    };

    http.Response response = await http.post(
      '${this._baseUrl}/$method',
      body: body,
    );

    return jsonDecode(response.body);
  }

  /// Execute [code]
  Future<Map> execute(String code) async {
    return this.call('execute', {
      'code': code,
    });
  }
}