import 'dart:convert';
import 'dart:math';

import 'package:http/http.dart' as http;

import './apiexception.dart';

Random random = new Random();

/// VK API Messages class
class Messages {
  /// Base API request url
  String _baseUrl = 'https://api.vk.com/method/messages';
  
  /// Group's or user's token
  String _token;

  /// API version
  String _version = '5.98';

  /// Messages
  Messages(String token) {
    this._token = token;
  }

  toString() {
    return 'Messages';
  }

  /// Send a [method] request with [params]
  Future<Map> _request(String method, [Map<String, String> params]) async {
    String url = this._baseUrl + '.$method';

    Map body = <String, String>{
      'access_token': this._token,
      'v': this._version,
      ...?params,
    };

    http.Response response = await http.post(
      url,
      body: body,
    );

    Map json = jsonDecode(response.body);

    if (json['error'] != null) {
      Map error = json['error'];

      throw new APIException(
        error['error_msg'],
        code: error['error_code'],
      ).error;
    }

    return json;
  }

  /// Add chat user
  Future<Map> addChatUser([Map<String, String> params]) async {
    return this._request('addChatUser', {
      ...?params,
    });
  }

  /// Allow messages from group
  Future<Map> allowMessagesFromGroup([Map<String, String> params]) async {
    return this._request('allowMessagesFromGroup', {
      ...?params,
    });
  }

  /// Create chat
  Future<Map> createChat([Map<String, String> params]) async {
    return this._request('createChat', {
      ...?params,
    });
  }

  /// Delete message
  Future<Map> delete([Map<String, String> params]) async {
    return this._request('delete', {
      ...?params,
    });
  }

  /// Delete chat photo
  Future<Map> deleteChatPhoto([Map<String, String> params]) async {
    return this._request('deleteChatPhoto', {
      ...?params,
    });
  }

  /// Delete conversation
  Future<Map> deleteConversation([Map<String, String> params]) async {
    return this._request('deleteConversation', {
      ...?params,
    });
  }

  /// Deny messages from group
  Future<Map> denyMessagesFromGroup([Map<String, String> params]) async {
    return this._request('denyMessagesFromGroup', {
      ...?params,
    });
  }

  /// Edit message
  Future<Map> edit([Map<String, String> params]) async {
    return this._request('edit', {
      ...?params,
    });
  }

  /// Edit chat name
  Future<Map> editChat([Map<String, String> params]) async {
    return this._request('editChat', {
      ...?params,
    });
  }

  /// Get message by conversation message ID
  Future<Map> getByConversationMessageId([Map<String, String> params]) async {
    return this._request('getByConversationMessageId', {
      ...?params,
    });
  }

  /// Get message by ID
  Future<Map> getById([Map<String, String> params]) async {
    return this._request('getById', {
      ...?params,
    });
  }

  /// Get chat info
  Future<Map> getChat([Map<String, String> params]) async {
    return this._request('getChat', {
      ...?params,
    });
  }

  /// Get chat preview
  Future<Map> getChatPreview([Map<String, String> params]) async {
    return this._request('getChatPreview', {
      ...?params,
    });
  }

  /// Get conversation members
  Future<Map> getConversationMembers([Map<String, String> params]) async {
    return this._request('getConversationMembers', {
      ...?params,
    });
  }

  /// Get conversations
  Future<Map> getConversations([Map<String, String> params]) async {
    return this._request('getConversations', {
      ...?params,
    });
  }

  /// Get conversations by ID
  Future<Map> getConversationsById([Map<String, String> params]) async {
    return this._request('getConversationsById', {
      ...?params,
    });
  }

  /// Gey messages history
  Future<Map> getHistory([Map<String, String> params]) async {
    return this._request('getHistory', {
      ...?params,
    });
  }

  /// Get history attachments
  Future<Map> getHistoryAttachments([Map<String, String> params]) async {
    return this._request('getHistoryAttachments', {
      ...?params,
    });
  }

  /// Get important messages
  Future<Map> getImportantMessages([Map<String, String> params]) async {
    return this._request('getImportantMessages', {
      ...?params,
    });
  }

  /// Get invite link
  Future<Map> getInviteLink([Map<String, String> params]) async {
    return this._request('getInviteLink', {
      ...?params,
    });
  }

  /// Get last activity
  Future<Map> getLastActivity([Map<String, String> params]) async {
    return this._request('getLastActivity', {
      ...?params,
    });
  }

  /// Get longpoll history
  Future<Map> getLongPollHistory([Map<String, String> params]) async {
    return this._request('getLongPollHistory', {
      ...?params,
    });
  }

  /// Get longpoll server
  Future<Map> getLongPollServer([Map<String, String> params]) async {
    return this._request('getLongPollServer', {
      ...?params,
    });
  }

  /// Is messages from group allowed?
  Future<Map> isMessagesFromGroupAllowed([Map<String, String> params]) async {
    return this._request('isMessagesFromGroupAllowed', {
      ...?params,
    });
  }

  /// Join chat by invite link
  Future<Map> joinChatByInviteLink([Map<String, String> params]) async {
    return this._request('joinChatByInviteLink', {
      ...?params,
    });
  }

  /// Mark conversation as answered
  Future<Map> markAsAnsweredConversation([Map<String, String> params]) async {
    return this._request('markAsAnsweredConversation', {
      ...?params,
    });
  }

  /// Mark message as important
  Future<Map> markAsImportant([Map<String, String> params]) async {
    return this._request('markAsImportant', {
      ...?params,
    });
  }

  /// Mark conversation as important
  Future<Map> markAsImportantConversation([Map<String, String> params]) async {
    return this._request('markAsImportantConversation', {
      ...?params,
    });
  }

  /// Mark message as read
  Future<Map> markAsRead([Map<String, String> params]) async {
    return this._request('markAsRead', {
      ...?params,
    });
  }

  /// Pin message
  Future<Map> pin([Map<String, String> params]) async {
    return this._request('pin', {
      ...?params,
    });
  }

  /// Remove user from chat
  Future<Map> removeChatUser([Map<String, String> params]) async {
    return this._request('removeChatUser', {
      ...?params,
    });
  }

  /// Restore message
  Future<Map> restore([Map<String, String> params]) async {
    return this._request('restore', {
      ...?params,
    });
  }

  /// Search messages
  Future<Map> search([Map<String, String> params]) async {
    return this._request('search', {
      ...?params,
    });
  }

  /// Search conversations
  Future<Map> searchConversations([Map<String, String> params]) async {
    return this._request('searchConversations', {
      ...?params,
    });
  }

  /// Send a message
  Future<Map> send([Map<String, String> params]) async {
    return this._request('send', {
      'random_id': random.nextInt(2147483647).toString(),
      ...?params,
    });
  }

  /// Set activity
  Future<Map> setActivity([Map<String, String> params]) async {
    return this._request('setActivity', {
      ...?params,
    });
  }

  /// Set chat photo
  Future<Map> setChatPhoto([Map<String, String> params]) async {
    return this._request('setChatPhoto', {
      ...?params,
    });
  }

  /// Unpin message
  Future<Map> unpin([Map<String, String> params]) async {
    return this._request('unpin', {
      ...?params,
    });
  }
}